/******************************************************************************
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:        Lock_Transforms.ms
Date:           1/18/12
Description:    This script will lock the specified transforms of the selected
                objects
******************************************************************************/

try (destroyDialog lock_transforms_rollout) catch()


struct LockTransforms
(
    /**************************************************************************
    Adds the 'lock_states' to the objects current lock set
    **************************************************************************/
    fn lock_objects object_list lock_states =
    (
        for current_object in object_list do
        (
            locks = (copy lock_states) + (getTransformLockFlags current_object)
                
            setTransformLockFlags current_object locks
        ) --end for
    ), --end fn
    
    /**************************************************************************
    Overrides the current lock set with 'lock_states'
    **************************************************************************/
    fn override_locks object_list lock_states =
    (
        for current_object in object_list do
            setTransformLockFlags current_object lock_states
    ), --end fn
    
    /**************************************************************************
    Unlocks the 'lock_states' from the object current lock set
    **************************************************************************/
    fn unlock_objects object_list lock_states =
    (
        for current_object in object_list do
        (
            locks = (getTransformLockFlags current_object) - lock_states
                
            setTransformLockFlags current_object locks
        ) --end for
    ), --end fn
    
    /**************************************************************************
    Sets the wirecolor color for all objects in 'object_list' NOTE: VRay lights
    have special wirecolor properties
    **************************************************************************/
    fn set_wirecolor object_list current_color =
    (
        for current in object_list do
        (
            --vray lights have special wirecolor settings
            if isProperty current "viewport_wire_color" then
            (
                current.viewport_wire_color_on = true
                current.viewport_wire_color = current_color
            ) --end if
            else
                current.wirecolor = current_color
        ) --end for
    ) --end fn
) --end struct


( --put script into local scope
    global lock_transforms_rollout
    local lock_transforms_ini = getDir #plugcfg + "\\LockTransforms.ini"
    local lock_transforms_position
        
    rollout lock_transforms_rollout "Lock Transforms"
    (
        local locker = LockTransforms()
        
        groupBox mode_group "Mode" pos:[5,5] width:165 height:65
            radioButtons mode_radio "" pos:[15,20] height:16 labels:#("Lock", "Lock and Unlock", "Unlock") columns:1 default:1
        
        groupBox lock_group "Locks" pos:[5,75] width:165 height:105
            label move_label"Move:" pos:[15,95] width:35 height:15
            checkbox move_x_checkbox "X" pos:[65,95] width:30 height:15
            checkbox move_y_checkbox "Y" pos:[100,95] width:30 height:15
            checkbox move_z_checkbox "Z" pos:[135,95] width:30 height:15
        
            label rotate_label "Rotate:" pos:[15,115] width:35 height:15
            checkbox rotate_x_checkbox "X" pos:[65,115] width:30 height:15
            checkbox rotate_y_checkbox "Y" pos:[100,115] width:30 height:15
            checkbox rotate_z_checkbox "Z" pos:[135,115] width:30 height:15
        
            label scale_label "Scale:" pos:[15,135] width:35 height:15
            checkbox scale_x_checkbox "X" pos:[65,135] width:30 height:15
            checkbox scale_y_checkbox "Y" pos:[100,135] width:30 height:15
            checkbox scale_z_checkbox "Z" pos:[135,135] width:30 height:15
        
            button check_all_button "All" pos:[15,160] width:40 height:15
            button check_none_button "None" pos:[65,160] width:40 height:15
            button check_invert_button "Invert" pos:[115,160] width:40 height:15
        
        groupBox option_group "Options" pos:[5,185] width:165 height:40
            checkbox wirecolor_checkbox "Set Wirecolor" pos:[15,205] width:85 height:15
            colorPicker wirecolor_color "" pos:[110,205] width:40 height:15
        
        button lock_selection_button "Lock Selected Objects" pos:[5,235] width:165 height:20
        label status_label "INFO: Standing by..." pos:[5,265] width:165 height:15
        
        /**********************************************************************
        UI open event handler. Prepares the UI for first time open
        **********************************************************************/
        on lock_transforms_rollout open do
        (
            local current_color = execute (getIniSetting lock_transforms_ini "Dialog" "Wirecolor")
            
            if current_color == OK do
                current_color = color 50 50 50
            
            wirecolor_color.color = current_color
        ) --end on
            
        /**********************************************************************
        UI move event handler. Stores the curent position of the UI
        **********************************************************************/
        on lock_transforms_rollout moved pos do
            setIniSetting lock_transforms_ini "Dialog" "Position" (pos as string)
        
        /**********************************************************************
        UI close event handler. Saves the current wirecolor for next open
        **********************************************************************/
        on lock_transforms_rollout close do
            setIniSetting lock_transforms_ini "Dialog" "Wirecolor" (wirecolor_color.color as string)
        
        /**********************************************************************
        Mode changed event handler. Sets the text of the execute button
        **********************************************************************/
        on mode_radio changed the_state do
        (
            if the_state < 3 then
                lock_selection_button.text = "Lock Selected Objects"
            else
                lock_selection_button.text = "Unlock Selected Objects"
        ) --end on
        
        /**********************************************************************
        Check all event handler. Sets all the move lock checkboxs states to true
        **********************************************************************/
        on check_all_button pressed do
        (
            move_x_checkbox.state = true
            move_y_checkbox.state = true
            move_z_checkbox.state = true
            rotate_x_checkbox.state = true
            rotate_y_checkbox.state = true
            rotate_z_checkbox.state = true
            scale_x_checkbox.state = true
            scale_y_checkbox.state = true
            scale_z_checkbox.state = true
        ) --end on
        
        /**********************************************************************
        Check none event handler. Sets all the move lock checkboxs states to false
        **********************************************************************/
        on check_none_button pressed do
        (
            move_x_checkbox.state = false
            move_y_checkbox.state = false
            move_z_checkbox.state = false
            rotate_x_checkbox.state = false
            rotate_y_checkbox.state = false
            rotate_z_checkbox.state = false
            scale_x_checkbox.state = false
            scale_y_checkbox.state = false
            scale_z_checkbox.state = false
        ) --end on
        
        /**********************************************************************
        Check invert event handler. Sets all the move lock checkboxs states to
        their opposite states
        **********************************************************************/
        on check_invert_button pressed do
        (
            move_x_checkbox.state = not(move_x_checkbox.state)
            move_y_checkbox.state = not(move_y_checkbox.state)
            move_z_checkbox.state = not(move_z_checkbox.state)
            rotate_x_checkbox.state = not(rotate_x_checkbox.state)
            rotate_y_checkbox.state = not(rotate_y_checkbox.state)
            rotate_z_checkbox.state = not(rotate_z_checkbox.state)
            scale_x_checkbox.state = not(scale_x_checkbox.state)
            scale_y_checkbox.state = not(scale_y_checkbox.state)
            scale_z_checkbox.state = not(scale_z_checkbox.state)
        ) --end on
        
        /**********************************************************************
        Lock selection event handler. Sets the locks for the selected objects
        **********************************************************************/
        on lock_selection_button pressed do
        (
            local lock_states = #{}
            
            if move_x_checkbox.state do append lock_states 1
            if move_y_checkbox.state do append lock_states 2
            if move_z_checkbox.state do append lock_states 3
            if rotate_x_checkbox.state do append lock_states 4
            if rotate_y_checkbox.state do append lock_states 5
            if rotate_z_checkbox.state do append lock_states 6
            if scale_x_checkbox.state do append lock_states 7
            if scale_y_checkbox.state do append lock_states 8
            if scale_z_checkbox.state do append lock_states 9
            
            case mode_radio.state of
            (
                (1): locker.lock_objects selection lock_states
                (2): locker.override_locks selection lock_states
                (3): locker.unlock_objects selection lock_states
            ) --end case
            
            if wirecolor_checkbox.checked do
                locker.set_wirecolor selection wirecolor_color.color
            
            message = ("INFO: " + (selection.count as string) + " object(s) ")
            
            if mode_radio.state < 3 then
                message += "locked"
            else
                message += "unlocked"
            
            status_label.text = message
        ) --end on
    ) --end rollout
    
    lock_transforms_position = execute (getIniSetting lock_transforms_ini "Dialog" "Position")
    if lock_transforms_position == OK do lock_transforms_position = [100,100]
    if lock_transforms_position.x > sysinfo.desktopsize.x - 175 do
        lock_transforms_position.x = sysinfo.desktopsize.x - 180
    if lock_transforms_position.y > sysinfo.desktopsize.y - 340 do
        lock_transforms_position.y = sysinfo.desktopsize.y - 340
    if lock_transforms_position.x < 0 do lock_transforms_position.x = 0 
    if lock_transforms_position.y < 0 do lock_transforms_position.y = 0
    createDialog lock_transforms_rollout 175 280 lock_transforms_position.x lock_transforms_position.y
) --end local scope
